--    Copyright (C) 2004 by neuron                                          
--    aagaande@gmail.com                                            
--                                                                          
--    This program is free software; you can redistribute it and--or modify  
--    it under the terms of the GNU General Public License as published by  
--    the Free Software Foundation; either version 2 of the License, or     
--    (at your option) any later version.                                   
--                                                                          
--    This program is distributed in the hope that it will be useful,       
--    but WITHOUT ANY WARRANTY; without even the implied warranty of        
--    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         
--    GNU General Public License for more details.                          
--                                                                          
--    You should have received a copy of the GNU General Public License     
--    along with this program; if not, write to the                         
--    Free Software Foundation, Inc.,                                       
--    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 

local options = {
		type='group',
		args = {
	         resetPos = {
	            type = 'execute',
	            name = 'Reset position of tracker',
	            desc = 'This is incase you lost it',
	            func = function()
	            	nQuestLogFrame:ClearAllPoints()
	            	nQuestLogFrame:SetPoint("CENTER",0,0)
	            	nQuestLog:SavePosition()
	            	nQuestLog:SetPosition()
	            	nQuestLog.db.profile["trackerLocked"] = false
	            end
	        },
	         toggleVisible = {
	            type = 'toggle',
	            name = 'Tracker Visible',
	            desc = 'Note that tracker will not show if your not tracking any quests.',
	            get = function()
	            	return nQuestLog.db.char["visible"]
	            end,
	            set = function(v)
	            	nQuestLog.db.char["visible"] = v
	            	nQuestLog:ToggleVisible(v)
	            end
	        },
	        spacer = {
	        	type = 'header',
	        	name = ' ',
	        	order = 110,
	        },
	        settings = {
	        	type = 'group',
	        	order = 90,
	        	name = 'Settings',
	        	desc = 'This needs work, I know',
	        	args = {}
	        }
		}
	}
	
local function ToggleMenu(data)
	local func_Get,func_Set
	
	if (data.type == "toggle" or data.type == "range" or data.type == "text") then
		func_Get = function()
			if (data.reversed) then
				return not nQuestLog.db.profile[data.variable]
			end
			return nQuestLog.db.profile[data.variable]
		end
		func_Set = function(v)
			if (data.reversed) then
				v = not v
			end
			nQuestLog.db.profile[data.variable] = v
			nQuestLog:Refresh()
			nQuestLog:UpdateEvents()
			
			-- Custom stuff needed for only few things:
			if (data.variable == "width") then
				-- This refreshes width:
				nQuestLog:ToggleVisible(true)
			elseif (data.variable == "trackerGrowUp") then
				nQuestLog:SavePosition()
				nQuestLog:SetPosition()
			end
		end
	elseif(data.type == "color") then
		func_Get = function()
			-- nQuestLog:Print("Returning " .. data.value.r .. " " .. data.value.g .. " " .. data.value.b)
			if (data.hasAlpha) then
				return data.value.r / 255,data.value.g / 255,data.value.b / 255, data.value.a
			end
			return data.value.r / 255,data.value.g / 255,data.value.b / 255
		end
		func_Set = function(r,g,b,a)
			-- nQuestLog:Print("Setting " .. r*255 .. " " .. g*255 .. " " .. b*255)
			nQuestLog.db.profile[data.variable] = {r=r * 255,g=g * 255,b=b * 255,a=a}
			nQuestLog:Refresh()
		end
	end
	
	return func_Get,func_Set	
end

local AceProfile = {
	positionBottom = -1,
	positionLeft = -1,
	positionTop = -1,
}
local settingsTable = {
	-- This table is only used to generate AceProfile and the menu to change it
	{ name="Main settings", type="header", variable="MainHeader" },
	{ name="Lock tracker", type="toggle", desc="Lock the tracker to avoid dragging (drag by pulling top left)", variable="trackerLocked", value=false },
	{ name="Modify tooltip", type="toggle", desc="Show tooltip information when mousing over mob/item", variable="showToolTip", value=true },
	
	{ name="Quest tracking", type="group" },
	{ name="Track new quests", type="toggle", desc="Auto track new quests", variable="trackNewQuests", value=true },
	{ name="Track when looting questitem", type="toggle", desc="Auto track hidden quests when you loot a quest item for it", variable="trackWhenLooting", value=true },
	
	-- Quest objective tracking related
	{ name="Quest objectives tracking", type="group" },
	{ name="Normal objective tracking", type="toggle", desc="Track quest objectives the old fashion way", variable="smartObjectiveTracking", value=true, reversed=true },
		{ name="  -  Hide All Objectives", type="toggle", desc="Hide all quest objectives", variable="hideAllObjectives", value=true, disabled=function() return nQuestLog.db.profile.smartObjectiveTracking end },
		{ name="  -  Hide Completed Objectives", type="toggle", desc="Hide completed objectives", variable="hideCompletedObjectives", value=true, disabled=function() return nQuestLog.db.profile.smartObjectiveTracking end },
	
	{ name="Smart quest objective tracking", type="toggle", desc="Enable smart quest objective tracking\nThis will automatically show/hide quest objectives based on time/what your already tracking.", variable="smartObjectiveTracking", value=true },
	{ name="  -  Max objectives tracked", type="range", desc="Max quest objectives to track", variable="smartObjectiveTracking_Max", value=5, min=1,max=20,step=1, disabled=function() return not nQuestLog.db.profile.smartObjectiveTracking end },
	{ name="  -  Max age of objective", type="range", desc="Max age of a quest objective to keep tracking it (in minutes)", variable="smartObjectiveTracking_MaxAge", value=30, min=1,max=120,step=1, disabled=function() return not nQuestLog.db.profile.smartObjectiveTracking end },
	{ name="  -  Hide objectives for new quests", type="toggle", desc="Hide objectives for new quests", variable="hideAllObjectives", value=true, disabled=function() return not nQuestLog.db.profile.smartObjectiveTracking end },
	{ name="  -  Hide completed objectives", type="toggle", desc="Hide completed objectives", variable="hideCompletedObjectives", value=true, disabled=function() return not nQuestLog.db.profile.smartObjectiveTracking end },

	-- Alert specific options
	{ name="Notification options", type="group"},
	{ name="Notify on looted questitem", type="toggle", desc="Notify you in chatframe when you loot a quest item", variable="alertLootedQuestItem", value=true },
	{ name="  -  Only when the quest is hidden", type="toggle", desc="Only notify if the quest is hidden", variable="alertLootedQuestItemHiddenOnly", value=true },
	{ name="Notify on finished objective", type="toggle", desc="Notify you when you've finished a quest objective", variable="alertFinishedObjective", value=false },

	

	{ name="Layouts", type="header", variable="LayoutHeader",menuParent=options.args.settings.args  },
	
	-- Layout stuff:
		{ name="Tracker layout", type="group" },
		{ name="Background transparency", type="range", desc="Set the alpha transparency of the tracker", variable="trackerTransparency", value=.75,isPercent=true },
		{ name="Border", type="color", desc="Set border color and transparency", variable="trackerBorderColor", value={r=255,g=255,b=255,a=1},hasAlpha=true },
		{ name="Grow upwards", type="toggle", desc="Grow tracker up", variable="trackerGrowUp", value=false },
		{ name="Tooltip anchor", type="text", desc="Decides location for the tooltip in the quest tracker.", variable="tooltipAnchor", value="DEFAULT", validate={"DEFAULT","ANCHOR_TOPRIGHT", "ANCHOR_RIGHT", "ANCHOR_BOTTOMRIGHT","ANCHOR_TOPLEFT","ANCHOR_LEFT","ANCHOR_BOTTOMLEFT"} },
		{ name="Width", type="range", desc="Set width of frame", variable="width", value=266, min=50,max=500 },
	
		--Zone Related:
		{ name="Zone layout", type="group" },
		{ name="Show empty zones", type="toggle", desc="Show hidden zones where you are not tracking any quests", variable="showEmptyZones", value=false },
		{ name="Zone header color", type="color", desc="Set color of zone header", variable="zoneHeader", value={r=119,g=187,b=255} },
		{ name="  -  Color by level", type="toggle", desc="Color zone name by zone level", variable="colorZoneByLevel", value=false },
		{ name="Show Zone level", type="toggle", desc="Show zone level", variable="showZoneLevel", value=true },
		{ name="  -  Color by level", type="toggle", desc="Color zone's level text by zone level", variable="colorZoneLevelByLevel", value=false },
		
		--Quest Related:
		{ name="Quest layout", type="group" },
		{ name="Quest header color", type="color", desc="Set color of quest header", variable="questHeader", value={r=255,g=255,b=255} },
		{ name="  -  Color by level", type="toggle", desc="Color quest name by level", variable="colorQuestByLevel", value=true },
		{ name="Show Quest Progress", type="toggle", desc="Show quest progress (1/7 items total)", variable="showQuestProgress", value=true },
		{ name="  -  Color by level", type="toggle", desc="Color zone's level text by zone level", variable="colorQuestProgressByProgress", value=true },
		{ name="Hilight zone quests", type="toggle", desc="Scan objectives for quests and hilight quests for current zone", variable="showQuestHilight", value=true },
		
		--Quest objectives related:
		{ name="Quest objectives layout", type="group" },
		{ name="Show Description when no objectives", type="toggle", desc="Show small quest description when no progress is available", variable="showShortWhenNoObjectives", value=true },
		{ name="Color Objective By Progress", type="toggle", desc="Color objective text by progress", variable="colorObjectiveByProgress", value=false },
		{ name="Color Objective Progress By Progress", type="toggle", desc="Color objective progress by progress", variable="colorObjectiveProgressByProgress", value=true },
}
	
-- Make options menu for all settings:
local function MakeAllSettings()
	local menuParent = options.args.settings.args
	for i,data in ipairs(settingsTable) do
		local func_Get,func_Set = ToggleMenu(data)
	
		-- First add menu
		local strName = data.name
		if (data.type ~= "header") then
			strName = "  " .. strName
		end
		
		if (data.type == "group") then
			menuParent = options.args.settings.args
			data.desc = strName .. " settings"
			data.variable = data.name
			data.args = {}
		end
		
		if (data.menuParent) then
			menuParent = data.menuParent
		end
		
		menuParent[data.name] = {
	        type = data.type,
	        name = strName,
	        desc = data.desc, 
	        order = i,
	        get = func_Get,
	        set = func_Set,
	        min = data["min"],
	        max = data["max"],
	        step = data["step"],
	        isPercent = data["isPercent"],
	        validate = data["validate"],
	        args = data.args,
	        disabled = data.disabled,
	        hasAlpha = hasAlpha,
		}
		
		if (data.type == "group") then
			menuParent = menuParent[data.variable].args
		end
		
		--Then add us to the RegisterDefaults db.
		AceProfile[data.variable] = data.value
	end
	
	settingsTable = nil
end

MakeAllSettings()


nQuestLog:RegisterDB("nQuestLogDB")
nQuestLog:RegisterDefaults('profile', AceProfile)

nQuestLog:RegisterDefaults('char', {
	visible = true,
	-- TODO IMPORTANT, auto clean watchedQuests for quests that no longer exists (do it on login)
	watchedQuests = {},
	hiddenZones = {},
})

nQuestLog.mainMenu = options
